@@ -49,9 +49,6 @@
 /* All drivers initialising the SW cursor need this */
 #include "mipointer.h"
 
-/* All drivers implementing backing store need this */
-#include "mibstore.h"
-
 /* Colormap handling */
 #include "micmap.h"
 #include "xf86cmap.h"
@@ -64,7 +61,7 @@
 #define DPMS_SERVER
 #include <X11/extensions/dpms.h>
 #endif
-
+#include "compat-api.h"
 
 /* Mandatory functions */
 static const OptionInfoRec * VESAAvailableOptions(int chipid, int busid);
@@ -75,17 +72,16 @@ static Bool VESAPciProbe(DriverPtr drv, int entity_num,
      struct pci_device *dev, intptr_t match_data);
 #endif
 static Bool VESAPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool VESAScreenInit(int Index, ScreenPtr pScreen, int argc,
-			   char **argv);
-static Bool VESAEnterVT(int scrnIndex, int flags);
-static void VESALeaveVT(int scrnIndex, int flags);
-static Bool VESACloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool VESAScreenInit(SCREEN_INIT_ARGS_DECL);
+static Bool VESAEnterVT(VT_FUNC_ARGS_DECL);
+static void VESALeaveVT(VT_FUNC_ARGS_DECL);
+static Bool VESACloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool VESASaveScreen(ScreenPtr pScreen, int mode);
 
-static Bool VESASwitchMode(int scrnIndex, DisplayModePtr pMode, int flags);
+static Bool VESASwitchMode(SWITCH_MODE_ARGS_DECL);
 static Bool VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode);
-static void VESAAdjustFrame(int scrnIndex, int x, int y, int flags);
-static void VESAFreeScreen(int scrnIndex, int flags);
+static void VESAAdjustFrame(ADJUST_FRAME_ARGS_DECL);
+static void VESAFreeScreen(FREE_SCREEN_ARGS_DECL);
 static void VESAFreeRec(ScrnInfoPtr pScrn);
 static VESAPtr VESAGetRec(ScrnInfoPtr pScrn);
 
@@ -111,7 +107,7 @@ static void *
 VESAWindowLinear(ScreenPtr pScreen, CARD32 row, CARD32 offset, int mode,
 		 CARD32 *size, void *closure)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
 
     *size = pVesa->maxBytesPerScanline;
@@ -122,7 +118,7 @@ static void *
 VESAWindowWindowed(ScreenPtr pScreen, CARD32 row, CARD32 offset, int mode,
 		   CARD32 *size, void *closure)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
     VbeModeInfoBlock *data = ((VbeModeInfoData*)(pScrn->currentMode->Private))->data;
     int window;
@@ -342,11 +338,11 @@ vesaModesCloseEnough(DisplayModePtr edid, DisplayModePtr vbe)
 }
 
 static ModeStatus
-VESAValidMode(int scrn, DisplayModePtr p, Bool flag, int pass)
+VESAValidMode(SCRN_ARG_TYPE arg, DisplayModePtr p, Bool flag, int pass)
 {
+    SCRN_INFO_PTR(arg);
     static int warned = 0;
     int found = 0;
-    ScrnInfoPtr pScrn = xf86Screens[scrn];
     VESAPtr pVesa = VESAGetRec(pScrn);
     MonPtr mon = pScrn->monitor;
     ModeStatus ret = MODE_BAD;
@@ -357,7 +353,7 @@ VESAValidMode(int scrn, DisplayModePtr p, Bool flag, int pass)
 
     if (pass != MODECHECK_FINAL) {
 	if (!warned) {
-	    xf86DrvMsg(scrn, X_WARNING, "VESAValidMode called unexpectedly\n");
+	    xf86DrvMsg(pScrn->scrnIndex, X_WARNING, "VESAValidMode called unexpectedly\n");
 	    warned = 1;
 	}
 	return MODE_OK;
@@ -737,13 +733,11 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 
     if ((pScrn->monitor->DDC = pVesa->monitor) != NULL)
 	xf86SetDDCproperties(pScrn, pVesa->monitor);
-#ifdef HAVE_PANELID
     else {
 	void *panelid = VBEReadPanelID(pVesa->pVbe);
-	VBEInterpretPanelID(pScrn->scrnIndex, panelid);
+	VBEInterpretPanelID(SCRN_OR_INDEX_ARG(pScrn), panelid);
 	free(panelid);
     }
-#endif
 
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, DEBUG_VERB,
 			"Searching for matching VESA mode(s):\n");
@@ -842,16 +836,15 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pVesa->Options);
 
     /* Use shadow by default */
-    if (xf86ReturnOptValBool(pVesa->Options, OPTION_SHADOW_FB, TRUE)) 
-	pVesa->shadowFB = TRUE;
+    pVesa->shadowFB = xf86ReturnOptValBool(pVesa->Options, OPTION_SHADOW_FB,
+                                           TRUE);
 
     if (xf86ReturnOptValBool(pVesa->Options, OPTION_DFLT_REFRESH, FALSE))
 	pVesa->defaultRefresh = TRUE;
 
-    pVesa->ModeSetClearScreen = FALSE;
-    if (xf86ReturnOptValBool(pVesa->Options, OPTION_MODESET_CLEAR_SCREEN, 
-			     FALSE))
-	pVesa->ModeSetClearScreen = TRUE;
+    pVesa->ModeSetClearScreen =
+        xf86ReturnOptValBool(pVesa->Options,
+                             OPTION_MODESET_CLEAR_SCREEN, FALSE);
 
     if (!pVesa->defaultRefresh && !pVesa->strict_validation)
 	VBESetModeParameters(pScrn, pVesa->pVbe);
@@ -903,7 +896,7 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 static Bool
 vesaCreateScreenResources(ScreenPtr pScreen)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
     Bool ret;
 
@@ -918,19 +911,19 @@ vesaCreateScreenResources(ScreenPtr pScreen)
 }
 
 static void
-vesaEnableDisableFBAccess(int scrnIndex, Bool enable)
+vesaEnableDisableFBAccess(SCRN_ARG_TYPE arg, Bool enable)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     VESAPtr pVesa = VESAGetRec(pScrn);
 
     pVesa->accessEnabled = enable;
-    pVesa->EnableDisableFBAccess(scrnIndex, enable);
+    pVesa->EnableDisableFBAccess(arg, enable);
 }
 
 static Bool
-VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+VESAScreenInit(SCREEN_INIT_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
     VisualPtr visual;
     VbeModeInfoBlock *mode;
@@ -988,7 +981,7 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	return (FALSE);
 
     /* set the viewport */
-    VESAAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+    VESAAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
     /* Blank the screen for aesthetic reasons. */
     VESASaveScreen(pScreen, SCREEN_SAVER_ON);
@@ -1082,7 +1075,6 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     VESADGAInit(pScrn, pScreen);
 
     xf86SetBlackWhitePixels(pScreen);
-    miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
 
     /* software cursor */
@@ -1117,30 +1109,31 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 }
 
 static Bool
-VESAEnterVT(int scrnIndex, int flags)
+VESAEnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
 
     if (!VESASetMode(pScrn, pScrn->currentMode))
 	return FALSE;
-    VESAAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+    VESAAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
     return TRUE;
 }
 
 static void
-VESALeaveVT(int scrnIndex, int flags)
+VESALeaveVT(VT_FUNC_ARGS_DECL)
 {
-    VESASaveRestore(xf86Screens[scrnIndex], MODE_RESTORE);
+    SCRN_INFO_PTR(arg);
+    VESASaveRestore(pScrn, MODE_RESTORE);
 }
 
 static Bool
-VESACloseScreen(int scrnIndex, ScreenPtr pScreen)
+VESACloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
 
     if (pScrn->vtSema) {
-	VESASaveRestore(xf86Screens[scrnIndex], MODE_RESTORE);
+	VESASaveRestore(pScrn, MODE_RESTORE);
 	if (pVesa->savedPal)
 	    VBESetGetPaletteData(pVesa->pVbe, TRUE, 0, 256,
 				 pVesa->savedPal, FALSE, TRUE);
@@ -1160,21 +1153,21 @@ VESACloseScreen(int scrnIndex, ScreenPtr pScreen)
     pScrn->EnableDisableFBAccess = pVesa->EnableDisableFBAccess;
     pScreen->CreateScreenResources = pVesa->CreateScreenResources;
     pScreen->CloseScreen = pVesa->CloseScreen;
-    return pScreen->CloseScreen(scrnIndex, pScreen);
+    return pScreen->CloseScreen(CLOSE_SCREEN_ARGS);
 }
 
 static Bool
-VESASwitchMode(int scrnIndex, DisplayModePtr pMode, int flags)
+VESASwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     VESAPtr pVesa = VESAGetRec(pScrn);
     Bool ret, disableAccess = pVesa->ModeSetClearScreen && pVesa->accessEnabled;
 
     if (disableAccess)
-	pScrn->EnableDisableFBAccess(scrnIndex,FALSE);
-    ret = VESASetMode(xf86Screens[scrnIndex], pMode);
+        pScrn->EnableDisableFBAccess(SCRN_OR_INDEX_ARG(pScrn),FALSE);
+    ret = VESASetMode(pScrn, mode);
     if (disableAccess)
-	pScrn->EnableDisableFBAccess(scrnIndex,TRUE);
+	pScrn->EnableDisableFBAccess(SCRN_OR_INDEX_ARG(pScrn),TRUE);
     return ret;
 }
 
@@ -1235,17 +1228,19 @@ VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode)
 }
 
 static void
-VESAAdjustFrame(int scrnIndex, int x, int y, int flags)
+VESAAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-    VESAPtr pVesa = VESAGetRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    VESAPtr pVesa = VESAGetRec(pScrn);
 
     VBESetDisplayStart(pVesa->pVbe, x, y, TRUE);
 }
 
 static void
-VESAFreeScreen(int scrnIndex, int flags)
+VESAFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    VESAFreeRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    VESAFreeRec(pScrn);
 }
 
 static Bool
@@ -1578,7 +1573,7 @@ RestoreFonts(ScrnInfoPtr pScrn)
 static Bool
 VESASaveScreen(ScreenPtr pScreen, int mode)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
     Bool on = xf86IsUnblank(mode);
 
@@ -1603,7 +1598,7 @@ VESASaveScreen(ScreenPtr pScreen, int mode)
 static int 
 VESABankSwitch(ScreenPtr pScreen, unsigned int iBank)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VESAPtr pVesa = VESAGetRec(pScrn);
 
     if (pVesa->curBank == iBank)
@@ -1624,7 +1619,7 @@ VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
 {
     VESAPtr pVesa;
 
-    if (MODE_QUERY < 0 || function > MODE_RESTORE)
+    if (function < MODE_QUERY || function > MODE_RESTORE)
 	return (FALSE);
 
     pVesa = VESAGetRec(pScrn);
@@ -1735,9 +1730,9 @@ VESADGASetMode(ScrnInfoPtr pScrn, DGAModePtr pDGAMode)
 	frameY0 = pScrn->frameY0;
     }
 
-    if (!(*pScrn->SwitchMode)(scrnIdx, pMode, 0))
+    if (!(*pScrn->SwitchMode)(SWITCH_MODE_ARGS(pScrn, pMode)))
 	return (FALSE);
-    (*pScrn->AdjustFrame)(scrnIdx, frameX0, frameY0, 0);
+    (*pScrn->AdjustFrame)(ADJUST_FRAME_ARGS(pScrn, frameX0, frameY0));
 
     return (TRUE);
 }
@@ -1745,7 +1740,7 @@ VESADGASetMode(ScrnInfoPtr pScrn, DGAModePtr pDGAMode)
 static void
 VESADGASetViewport(ScrnInfoPtr pScrn, int x, int y, int flags)
 {
-    (*pScrn->AdjustFrame)(pScrn->pScreen->myNum, x, y, flags);
+    (*pScrn->AdjustFrame)(ADJUST_FRAME_ARGS(pScrn, x, y));
 }
 
 static int
